@@ -19,13 +19,17 @@ TRANSACTION_HEADER_SIZE = sum(field[1] for field in [
 	('reserved2', 4)
 ])
 
-AGGREGATE_HASHED_SIZE = sum(field[1] for field in [
+PRE_V3_AGGREGATE_HASHED_SIZE = sum(field[1] for field in [
 	('version_network_type', 4),
 	('max_fee', 8),
 	('deadline', 8),
 	('transactions_hash', Hash256.SIZE)
 ])
 
+AGGREGATE_HASHED_SIZE = PRE_V3_AGGREGATE_HASHED_SIZE + sum(field[1] for field in [
+	('payload_size', 4)
+])
+
 
 # region SymbolPublicAccount / SymbolAccount
 
@@ -59,6 +63,10 @@ class SymbolAccount(SymbolPublicAccount):
 		"""Cosigns a Symbol transaction."""
 		return self._facade.cosign_transaction(self.key_pair, transaction, detached)
 
+	def cosign_transaction_hash(self, transaction_hash, detached=False):
+		"""Cosigns a Symbol transaction hash."""
+		return self._facade.cosign_transaction_hash(self.key_pair, transaction_hash, detached)
+
 # endregion
 
 
@@ -116,22 +124,23 @@ class SymbolFacade:
 		hasher.update(self._transaction_data_buffer(transaction.serialize()))
 		return Hash256(hasher.digest())
 
-	def sign_transaction(self, key_pair, transaction):
-		"""Signs a Symbol transaction."""
+	def extract_signing_payload(self, transaction):
+		"""Gets the payload to sign given a Symbol transaction."""
 		sign_buffer = self.network.generation_hash_seed.bytes
 		sign_buffer += self._transaction_data_buffer(transaction.serialize())
-		return key_pair.sign(sign_buffer)
+		return sign_buffer
+
+	def sign_transaction(self, key_pair, transaction):
+		"""Signs a Symbol transaction."""
+		return key_pair.sign(self.extract_signing_payload(transaction))
 
 	def verify_transaction(self, transaction, signature):
 		"""Verifies a Symbol transaction."""
-		verify_buffer = self.network.generation_hash_seed.bytes
-		verify_buffer += self._transaction_data_buffer(transaction.serialize())
-		return Verifier(transaction.signer_public_key).verify(verify_buffer, signature)
-
-	def cosign_transaction(self, key_pair, transaction, detached=False):
-		"""Cosigns a Symbol transaction."""
-		transaction_hash = self.hash_transaction(transaction)
+		return Verifier(transaction.signer_public_key).verify(self.extract_signing_payload(transaction), signature)
 
+	@staticmethod
+	def cosign_transaction_hash(key_pair, transaction_hash, detached=False):
+		"""Cosigns a Symbol transaction hash."""
 		cosignature = sc.DetachedCosignature() if detached else sc.Cosignature()
 		if detached:
 			cosignature.parent_hash = sc.Hash256(transaction_hash.bytes)
@@ -141,6 +150,11 @@ class SymbolFacade:
 		cosignature.signature = sc.Signature(key_pair.sign(transaction_hash.bytes).bytes)
 		return cosignature
 
+	def cosign_transaction(self, key_pair, transaction, detached=False):
+		"""Cosigns a Symbol transaction."""
+		transaction_hash = self.hash_transaction(transaction)
+		return self.cosign_transaction_hash(key_pair, transaction_hash, detached)
+
 	@staticmethod
 	def hash_embedded_transactions(embedded_transactions):
 		"""Hashes embedded transactions of an aggregate."""
@@ -171,6 +185,7 @@ class SymbolFacade:
 		data_buffer_start = TRANSACTION_HEADER_SIZE
 		data_buffer_end = len(transaction_buffer)
 		if SymbolFacade._is_aggregate_transaction(transaction_buffer):
-			data_buffer_end = TRANSACTION_HEADER_SIZE + AGGREGATE_HASHED_SIZE
+			version = transaction_buffer[TRANSACTION_HEADER_SIZE]
+			data_buffer_end = TRANSACTION_HEADER_SIZE + (AGGREGATE_HASHED_SIZE if version >= 3 else PRE_V3_AGGREGATE_HASHED_SIZE)
 
 		return transaction_buffer[data_buffer_start:data_buffer_end]
